from typing import Optional, List, Dict, Self, Set

from edge.services.types.service import IService
from edge.types.imageref import ImageRef
from tracker.types.lrra import LrRaTrackset


class RaTrackFilterRuleConf:
    def __init__(self, mmr_classes: Set[str], max_speed: int):
        """
        :param mmr_classes:
            MMR classes
        :param max_speed:
            Max speed, km/h
        """
        self.mmr_classes: Set[str] = mmr_classes
        self.max_speed: int = max_speed


class LrTrackFilterRuleConf:
    def __init__(self, black_list: Set[str]):
        """
        :param black_list:
            templates of plates to be ignored
        """
        self.black_list: Set[str] = black_list


class RaTrackFilterConf:
    def __init__(self, rules: List[RaTrackFilterRuleConf]):
        """
        :param rules:
            List of active rules
        """
        self.rules = rules


class LrTrackFilterConf:
    def __init__(self, rules: List[LrTrackFilterRuleConf]):
        """
        :param rules:
            List of active rules
        """
        self.rules = rules


class TrackEmitterConf:
    def __init__(self, debounce_tout: int, event_dirs: List[str], ratrack_filter: RaTrackFilterConf, lrtrack_filter: LrTrackFilterConf):
        self.debounce_tout = debounce_tout
        self.event_dirs = event_dirs
        self.ratrack_filter = ratrack_filter
        self.lrtrack_filter = lrtrack_filter

    @classmethod
    def deserialize(cls, track_event_jnode: Dict) -> Self:
        debounce_tout = track_event_jnode.get('debounce-tout', 3)
        endpoints = track_event_jnode.get('endpoints', [])

        if not (0 <= debounce_tout <= 30):
            raise ValueError('main.conf#/fact-event/debounce-tout')

        ra_filters_jnode = track_event_jnode['ratrack-filter']

        ra_active_rules: List[RaTrackFilterRuleConf] = []
        for filter_jnode in ra_filters_jnode:
            if not filter_jnode.get('active', True):
                continue

            max_speed = filter_jnode['max-speed']
            mmr_classes = set(filter_jnode['mmr-class'])
            ra_active_rules.append(RaTrackFilterRuleConf(mmr_classes, max_speed))

        lr_filters_jnode = track_event_jnode.get('trackset-filter') or []
        lr_active_rules: List[LrTrackFilterRuleConf] = []
        if len(lr_filters_jnode) > 0:
            black_list = set(lr_filters_jnode)
            lr_active_rules.append(LrTrackFilterRuleConf(black_list))

        return cls(debounce_tout, endpoints, RaTrackFilterConf(ra_active_rules), LrTrackFilterConf(lr_active_rules))


class ITrackEmitter(IService):
    def consume_track(self, lrra_track: LrRaTrackset, imref: Optional[ImageRef]) -> None:
        raise NotImplementedError()
